@@ -45,6 +45,7 @@ static const char sccsid[] = "@(#)helix.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_helix
 #define PROGCLASS "Helix"
 #define HACK_INIT init_helix
 #define HACK_DRAW draw_helix
@@ -73,7 +74,7 @@ static XrmOptionDescRec opts[] =
 };
 static argtype vars[] =
 {
-	{(caddr_t *) & ellipse, (char *) "ellipse", (char *) "Ellipse", (char *) DEF_ELLIPSE, t_Bool}
+	{(void *) & ellipse, (char *) "ellipse", (char *) "Ellipse", (char *) DEF_ELLIPSE, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -184,7 +185,7 @@ trig(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
 	helixstruct *hp = &helixes[MI_SCREEN(mi)];
-	int         x1, y1, x2, y2;
+	int         x_1, y_1, x_2, y_2;
 	int         tmp, angle;
 
 
@@ -192,15 +193,15 @@ trig(ModeInfo * mi)
 
 	while (ABS(hp->d_angle) <= ANGLES) {
 		angle = hp->d_angle + hp->d_angle_offset;
-		x1 = (int) (sin_array[pmod(angle * hp->factor1, ANGLES)] * hp->xmid) +
+		x_1 = (int) (sin_array[pmod(angle * hp->factor1, ANGLES)] * hp->xmid) +
 			hp->xmid;
-		y1 = (int) (cos_array[pmod(angle * hp->factor1, ANGLES)] * hp->ymid) +
+		y_1 = (int) (cos_array[pmod(angle * hp->factor1, ANGLES)] * hp->ymid) +
 			hp->ymid;
-		x2 = (int) (sin_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
+		x_2 = (int) (sin_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
 			    hp->xmid) + hp->xmid;
-		y2 = (int) (cos_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
+		y_2 = (int) (cos_array[pmod(angle * hp->factor2 + hp->offset, ANGLES)] *
 			    hp->ymid) + hp->ymid;
-		XDrawLine(display, MI_WINDOW(mi), gc, x1, y1, x2, y2);
+		XDrawLine(display, MI_WINDOW(mi), gc, x_1, y_1, x_2, y_2);
 		if (MI_NPIXELS(mi) > 2) {
 			XSetForeground(display, gc, MI_PIXEL(mi, hp->color));
 			if (++hp->color >= MI_NPIXELS(mi))
@@ -354,7 +355,7 @@ void
 release_helix(ModeInfo * mi)
 {
 	if (helixes != NULL) {
-		(void) free((void *) helixes);
+		free(helixes);
 		helixes = (helixstruct *) NULL;
 	}
 }
