@@ -152,7 +152,6 @@ static const char sccsid[] = "@(#)life.c	5.07 2003/02/27 xlockmore";
 #define DEF_LABEL "True"
 #define FONT_HEIGHT 19
 #define FONT_WIDTH 15
-#define FONT_LENGTH 20
 #endif
 #define DEF_NEIGHBORS  "0"      /* choose best value (8) */
 #define DEF_SERIAL "False"
@@ -227,20 +226,20 @@ static XrmOptionDescRec opts[] =
 static argtype vars[] =
 {
 #ifdef LIFE_NAMES
-	{(caddr_t *) & label, (char *) "label", (char *) "Label", (char *) DEF_LABEL, t_Bool},
+	{(void *) & label, (char *) "label", (char *) "Label", (char *) DEF_LABEL, t_Bool},
 #endif
-	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
-	{(caddr_t *) & rule, (char *) "rule", (char *) "Rule", (char *) DEF_RULE, t_String},
-	{(caddr_t *) & lifefile, (char *) "lifefile", (char *) "LifeFile", (char *) "", t_String},
-	{(caddr_t *) & serial, (char *) "serial", (char *) "Serial", (char *) DEF_SERIAL, t_Bool},
-	{(caddr_t *) & conway, (char *) "conway", (char *) "Conway", (char *) DEF_CONWAY, t_Bool},
-	{(caddr_t *) & highlife, (char *) "highlife", (char *) "HighLife", (char *) DEF_HIGHLIFE, t_Bool},
-	{(caddr_t *) & daynight, (char *) "daynight", (char *) "DayNight", (char *) DEF_DAY_NIGHT, t_Bool},
-	{(caddr_t *) & callahan, (char *) "callahan", (char *) "Callahan", (char *) DEF_CALLAHAN, t_Bool},
-	{(caddr_t *) & andreen, (char *) "andreen", (char *) "Andreen", (char *) DEF_ANDREEN, t_Bool},
-	{(caddr_t *) & trilife, (char *) "trilife", (char *) "TriLife", (char *) DEF_TRILIFE, t_Bool},
-	{(caddr_t *) & trilife1, (char *) "trilife1", (char *) "TriLife1", (char *) DEF_TRILIFE1, t_Bool},
-	{(caddr_t *) & trilife2, (char *) "trilife2", (char *) "TriLife2", (char *) DEF_TRILIFE2, t_Bool}
+	{(void *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(void *) & rule, (char *) "rule", (char *) "Rule", (char *) DEF_RULE, t_String},
+	{(void *) & lifefile, (char *) "lifefile", (char *) "LifeFile", (char *) "", t_String},
+	{(void *) & serial, (char *) "serial", (char *) "Serial", (char *) DEF_SERIAL, t_Bool},
+	{(void *) & conway, (char *) "conway", (char *) "Conway", (char *) DEF_CONWAY, t_Bool},
+	{(void *) & highlife, (char *) "highlife", (char *) "HighLife", (char *) DEF_HIGHLIFE, t_Bool},
+	{(void *) & daynight, (char *) "daynight", (char *) "DayNight", (char *) DEF_DAY_NIGHT, t_Bool},
+	{(void *) & callahan, (char *) "callahan", (char *) "Callahan", (char *) DEF_CALLAHAN, t_Bool},
+	{(void *) & andreen, (char *) "andreen", (char *) "Andreen", (char *) DEF_ANDREEN, t_Bool},
+	{(void *) & trilife, (char *) "trilife", (char *) "TriLife", (char *) DEF_TRILIFE, t_Bool},
+	{(void *) & trilife1, (char *) "trilife1", (char *) "TriLife1", (char *) DEF_TRILIFE1, t_Bool},
+	{(void *) & trilife2, (char *) "trilife2", (char *) "TriLife2", (char *) DEF_TRILIFE2, t_Bool}
 };
 static OptionStruct desc[] =
 {
@@ -255,7 +254,7 @@ static OptionStruct desc[] =
 	{(char *) "-/+highlife", (char *) "turn on/off Bell's HighLife rule B36/S23"},
 	{(char *) "-/+daynight", (char *) "turn on/off Thompson's Day and Night rule B3678/S34678"},
 	{(char *) "-/+callahan", (char *) "turn on/off Callahan's hex rule B2a/S2b34"},
-	{(char *) "-/+andreen", (char *) "turn on/off Andreen's hex rule B2a3a4b/S2a2m4a"},
+	{(char *) "-/+andreen", (char *) "turn on/off Andreen's hex rule B2a3a4b/S2a2b4a"},
 	{(char *) "-/+trilife", (char *) "turn on/off Bay's tri rule B45/S34"},
 	{(char *) "-/+trilife1", (char *) "turn on/off Bay's tri rule B456/S45"},
 	{(char *) "-/+trilife2", (char *) "turn on/off Bay's tri rule B45/S23"}
@@ -335,7 +334,6 @@ typedef struct {
 	int         width, height;
 	int         state;
 	int         noChangeCount;
-	Bool        label;
 	int         redrawing, redrawpos;
 	int         ncells[STATES];
 	CellList   *last[STATES], *first[STATES];
@@ -448,7 +446,8 @@ codeToPatternedRule(int local_neighbors, paramstruct param)
 }
 
 static void
-copyFromPatternedRule(int local_neighbors,  paramstruct * param, int patterned_rule)
+copyFromPatternedRule(int local_neighbors, paramstruct * param,
+		int patterned_rule)
 {
 	int         i, neighbor_kind;
 
@@ -503,9 +502,11 @@ printRule(int local_neighbors, char * string, paramstruct param, Bool verbose)
 			i++;
 		} else if (l >= FIRSTGROUP && l < FIRSTGROUP + maxgroups[neighbor_kind]) {
 			for (g = 0; g < groupnumber[neighbor_kind][l - FIRSTGROUP]; g++) {
-				if (param.survival_group[l - FIRSTGROUP] & (1 << g))
-					(void) sprintf(&(string[i]), "%d%c", l, 'a' + g);
+				if (param.survival_group[l - FIRSTGROUP] & (1 << g)) {
+					(void) sprintf(&(string[i]), "%d%c",
+						l, 'a' + g);
 					i += 2;
+				}
 			}
 		}
 	}
@@ -517,9 +518,11 @@ printRule(int local_neighbors, char * string, paramstruct param, Bool verbose)
 			i++;
 		} else if (l >= FIRSTGROUP && l < FIRSTGROUP + maxgroups[neighbor_kind]) {
 			for (g = 0; g < groupnumber[neighbor_kind][l - FIRSTGROUP]; g++) {
-				if (param.birth_group[l - FIRSTGROUP] & (1 << g))
-					(void) sprintf(&(string[i]), "%d%c", l, 'a' + g);
+				if (param.birth_group[l - FIRSTGROUP] & (1 << g)) {
+					(void) sprintf(&(string[i]), "%d%c",
+						l, 'a' + g);
 					i += 2;
+				}
 			}
 		}
 	}
@@ -761,9 +764,10 @@ parseRule(ModeInfo * mi, char * string)
 	char        serving = 0;
 	static Bool found = False;
 
-	if (found)
+	if (!found)
+		lp->input_param.survival = lp->input_param.birth = 0;
+	if (lp->input_param.survival || lp->input_param.birth)
 		return;
-	lp->input_param.survival = lp->input_param.birth = 0;
 	for (n = 0; n < MAXGROUPS; n++) {
 		lp->input_param.survival_group[n] = lp->input_param.birth_group[n] = 0;
 	}
@@ -857,7 +861,7 @@ parseRule(ModeInfo * mi, char * string)
 		return;
 	}
 	neighbor_kind = invplot(lp->neighbors);
-	if (rule && !MI_IS_FULLRANDOM(mi)) {
+	if (rule) {
 		n = 0;
 		while (rule[n]) {
 			if (rule[n] == 'P' || rule[n] == 'p') {
@@ -903,7 +907,8 @@ parseRule(ModeInfo * mi, char * string)
 			n++;
 		}
 	}
-	if (!found) {		/* Default to Conway's rule if very stupid */
+	if (!found || !(lp->input_param.survival || lp->input_param.birth)) {
+		/* Default to Conway's rule if rule does not make sense */
 		lp->allGliders = True;
 		found = !MI_IS_FULLRANDOM(mi);
 		if (MI_IS_VERBOSE(mi))
@@ -1104,16 +1109,18 @@ draw_cell(ModeInfo * mi, cellstruct info)
  * PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 132 byte memory leak on
  * the next line */
 #ifdef XBM_GRELB
-		if (lp->logo2) {
+		    if (lp->logo2) {
 			(void) XPutImage(display, MI_WINDOW(mi), gc,
 				(LRAND() & 1) ? lp->logo : lp->logo2,
 			  0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
 					 lp->logo->width, lp->logo->height);
-		} else
+		    } else
 #endif
+		    {
 			(void) XPutImage(display, MI_WINDOW(mi), gc, lp->logo,
 			  0, 0, lp->xb + lp->xs * col, lp->yb + lp->ys * row,
 					 lp->logo->width, lp->logo->height);
+		    }
 		}
 	} else {		/* TRI */
 		int         orient = (col + row) % 2;	/* O left 1 right */
@@ -1261,8 +1268,12 @@ setcell(ModeInfo * mi, int col, int row, int state)
 	cellstruct  info;
 	CellList   *curr, *currn;
 
-	if (col < 0 || row < 0 || col >= lp->ncols || row >= lp->nrows)
+	if (col < 0 || row < 0 || col >= lp->ncols || row >= lp->nrows) {
+#ifdef DEBUG
+		(void) printf("col %d, row %d outside grid\n", col, row);
+#endif
 		return True; /* Actually its a 3rd case */
+	}
 
 	position = row * lp->ncols + col;
 	curr = lp->arr[position];
@@ -1391,7 +1402,7 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 	char       *patptr = (char *) NULL;
 #ifdef LIFE_NAMES
 	int pat = 2 * pattern + 1;
-	char * patstrg = "";
+	char * patstrg = (char *) "";
 #else
 	int pat = pattern;
 #endif
@@ -1421,26 +1432,26 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 			case 8:
 				switch (pattern_rule) {
 					case LIFE_8S23B3:
-						if (pat < common_8size) {
+						if (pattern < (int) common_8size) {
 						  patptr = &patterns_8S23B3_6[pat][0];
 #ifdef LIFE_NAMES
 						  patstrg = &patterns_8S23B3_6[2 * pattern][0];
 #endif
 						} else {
-						  patptr = &patterns_8S23B3[pat - 2 * common_8size][0];
+						  patptr = &patterns_8S23B3[pat - DIV * common_8size][0];
 #ifdef LIFE_NAMES
 						  patstrg = &patterns_8S23B3[2 * pattern - 2 * common_8size][0];
 #endif
 						}
 						break;
 					case LIFE_8S23B36:
-						if (pattern < common_8size) {
+						if (pattern < (int) common_8size) {
 						  patptr = &patterns_8S23B3_6[pat][0];
 #ifdef LIFE_NAMES
 						  patstrg = &patterns_8S23B3_6[2 * pattern][0];
 #endif
 						} else {
-						  patptr = &patterns_8S23B36[pat - 2 * common_8size][0];
+						  patptr = &patterns_8S23B36[pat - DIV * common_8size][0];
 #ifdef LIFE_NAMES
 						  patstrg = &patterns_8S23B36[2 * pattern - 2 * common_8size][0];
 #endif
@@ -1449,7 +1460,7 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 					case LIFE_8S34678B3678:
 						patptr = &patterns_8S34678B3678[pat][0];
 #ifdef LIFE_NAMES
-						  patstrg = &patterns_8S34678B3678[2 * pattern][0];
+						patstrg = &patterns_8S34678B3678[2 * pattern][0];
 #endif
 						break;
 				}
@@ -1471,7 +1482,7 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 					case LIFE_12S23B45:
 						patptr = &patterns_12S23B45[pat][0];
 #ifdef LIFE_NAMES
-						  patstrg = &patterns_12S23B45[2 * pattern][0];
+						patstrg = &patterns_12S23B45[2 * pattern][0];
 #endif
 						break;
 				}
@@ -1592,7 +1603,7 @@ GetPattern(ModeInfo * mi, int pattern_rule, int pattern)
 		row += lp->nrows / 2;
 		if (lp->neighbors % 3 == 0 && lp->neighbors != 6 &&
 				(lp->nrows / 2 + lp->ncols / 2 + 1) % 2) {
-			row++;	
+			row++;
 		}
 		SetList(col, row);
 	}
@@ -2058,7 +2069,7 @@ init_stuff(ModeInfo * mi)
 			xgcv.foreground = lp->black;
 			if ((lp->backGC = XCreateGC(display, window,
 					GCForeground | GCBackground,
-					&xgcv)) == NULL) {
+					&xgcv)) == None) {
 				free_life(display, lp);
 				return False;
 			}
@@ -2152,11 +2163,7 @@ init_life(ModeInfo * mi)
 			}
 		}
 	}
-	if (MI_IS_FULLRANDOM(mi))
-		lp->label = NRAND(3);
-	else
-		lp->label = label;
-	lp->labelOffsetY = NRAND(8);
+	lp->labelOffsetX = NRAND(8);
 	lp->labelOffsetY = NRAND(8);
 	parseRule(mi, lp->ruleString);
 	parseFile(mi);
@@ -2198,6 +2205,8 @@ init_life(ModeInfo * mi)
 			lp->param.birth_group[i] = lp->input_param.birth_group[i];
 		}
 		lp->patterned_rule = codeToPatternedRule(lp->neighbors, lp->param);
+		printRule(lp->neighbors, lp->ruleString, lp->param,
+			MI_IS_VERBOSE(mi));
 	}
 	lp->width = MI_WIDTH(mi);
 	lp->height = MI_HEIGHT(mi);
@@ -2226,9 +2235,12 @@ init_life(ModeInfo * mi)
 			lp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(lp->width, lp->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
 		else if (size < MINSIZE) {
-			if (!size)
-				lp->ys = MAX(MINSIZE, MIN(lp->width, lp->height) / (4 * MINGRIDSIZE));
-			else
+			if (!size) {
+				int min = MIN(lp->width, lp->height) / (4 * MINGRIDSIZE);
+				int max = MIN(lp->width, lp->height) / (2 * MINGRIDSIZE);
+
+				lp->ys = MAX(MINSIZE, min + NRAND(max - min + 1));
+			} else
 				lp->ys = MINSIZE;
 		} else
 			lp->ys = MIN(size, MAX(MINSIZE, MIN(lp->width, lp->height) /
@@ -2268,14 +2280,18 @@ init_life(ModeInfo * mi)
 				lp->pixelmode = False;
 				lp->xs = lp->logo->width;
 				lp->ys = lp->logo->height;
-			} else {
+			} else
+			{
+				int min = MIN(lp->width, lp->height) / (8 * MINGRIDSIZE);
+				int max = MIN(lp->width, lp->height) / (2 * MINGRIDSIZE);
+
+				lp->xs = lp->ys = MAX(MINSIZE, min + NRAND(max - min + 1));
 				lp->pixelmode = True;
-				lp->xs = lp->ys = MAX(MINSIZE, MIN(lp->width, lp->height) /
-						      MINGRIDSIZE);
 			}
 		}
+		else
 #endif
-		else {
+		{
 			lp->pixelmode = True;
 			if (size < -MINSIZE)
 				lp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(lp->width, lp->height) /
@@ -2304,9 +2320,12 @@ init_life(ModeInfo * mi)
 			lp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(lp->width, lp->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
 		else if (size < MINSIZE) {
-			if (!size)
-				lp->ys = MAX(MINSIZE, MIN(lp->width, lp->height) / (4 * MINGRIDSIZE));
-			else
+			if (!size) {
+				int min = MIN(lp->width, lp->height) / (4 * MINGRIDSIZE);
+				int max = MIN(lp->width, lp->height) / (MINGRIDSIZE);
+
+				lp->xs = lp->ys = MAX(MINSIZE, min + NRAND(max - min + 1));
+			} else
 				lp->ys = MINSIZE;
 		} else
 			lp->ys = MIN(size, MAX(MINSIZE, MIN(lp->width, lp->height) /
@@ -2506,8 +2525,9 @@ draw_life(ModeInfo * mi)
 				((MI_COUNT(mi) < 0) ? 1 : MI_COUNT(mi)) == 0)
 			shooter(mi);
 	}
-	if (lp->label) {
+	if (label) {
                 int size = MAX(MIN(MI_WIDTH(mi), MI_HEIGHT(mi)) - 1, 1);
+
                 if (size >= 10 * FONT_WIDTH) {
                         /* hard code these to corners */
                         XSetForeground(display, gc, MI_WHITE_PIXEL(mi));
@@ -2521,8 +2541,6 @@ draw_life(ModeInfo * mi)
                                 lp->nameString, strlen(lp->nameString));
                 }
          }
-
-
 }
 
 void
@@ -2628,7 +2646,8 @@ change_life(ModeInfo * mi)
 						lp->patterned_rule = 0;
 					break;
 			}
-			copyFromPatternedRule(lp->neighbors, &lp->param, lp->patterned_rule);
+			copyFromPatternedRule(lp->neighbors, &lp->param,
+				lp->patterned_rule);
 			printRule(lp->neighbors, lp->ruleString, lp->param,
 				MI_IS_VERBOSE(mi));
 		} else if (lp->allGliders) {
